==> acb-0.19.0.tar.gz: acb-0.19.0/tests/test_debug.py <==
from pathlib import Path
from typing import Any, Final
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from rich.console import Console
from acb.debug import (
    colorized_stderr_print,
    debug,
    init_debug,
    pprint,
    print_debug_info,
)
from acb.logger import Logger

TEST_MODULE_NAME: Final[str] = "test_module"
TEST_DEBUG_MSG: Final[str] = "test debug message"


@pytest.fixture
def mock_console() -> MagicMock:
    console = MagicMock(spec=Console)
    console.print = MagicMock()
    return console


@pytest.fixture
def mock_logger() -> MagicMock:
    logger = MagicMock(spec=Logger)
    logger.debug = MagicMock()
    return logger


class TestColorizedStderrPrint:
    def test_colorized_stderr_print_with_asyncio(self) -> None:
        with patch("acb.debug.colorize") as mock_colorize:
            mock_colorize.return_value = "Colored test message"

            with patch("acb.debug.supportTerminalColorsInWindows") as mock_support:
                mock_support.return_value.__enter__ = MagicMock()
                mock_support.return_value.__exit__ = MagicMock()

                mock_coro: AsyncMock = AsyncMock()
                with patch("acb.debug.aprint", return_value=mock_coro) as mock_aprint:

                    def mock_run_impl(coro: Any) -> None:
                        return None

                    with patch(
                        "acb.debug.asyncio.run",
                        side_effect=mock_run_impl,
                    ) as mock_run:
                        colorized_stderr_print("Test message")

                        mock_colorize.assert_called_once_with("Test message")

                        mock_support.assert_called_once()

                        mock_aprint.assert_called_once_with(
                            "Colored test message",
                            use_stderr=True,
                        )

                        mock_run.assert_called_once()

    def test_colorized_stderr_print_with_print(self) -> None:
        with patch("acb.debug.colorize") as mock_colorize:
            mock_colorize.return_value = "Colored test message"

            with patch("acb.debug.supportTerminalColorsInWindows") as mock_support:
                mock_support.return_value.__enter__ = MagicMock()
                mock_support.return_value.__exit__ = MagicMock()

                with (
                    patch(
                        "acb.debug.asyncio.run",
                        side_effect=Exception("Test exception"),
                    ),
                    patch("sys.stderr") as mock_stderr,
                ):
                    with patch("builtins.print") as mock_print:
                        colorized_stderr_print("Test message")

                        mock_print.assert_called_once_with(
                            "Colored test message",
                            file=mock_stderr,
                        )

    def test_colorized_stderr_print_with_no_color_support(self) -> None:
        with patch("acb.debug.colorize", side_effect=ImportError):
            with patch("builtins.print") as mock_print:
                with patch("sys.stderr") as mock_stderr:
                    colorized_stderr_print("Test message")
                    mock_print.assert_called_once_with("Test message", file=mock_stderr)


class TestPrintDebugInfo:
    def test_print_debug_info_with_module_not_deployed(self) -> None:
        mock_module: MagicMock = MagicMock(spec=Path)
        with patch("acb.debug.get_calling_module", return_value=mock_module):
            with patch("acb.debug._deployed", False):
                with patch("acb.debug.colorized_stderr_print") as mock_print:
                    result: Any = print_debug_info("Test message")

                    mock_print.assert_called_once_with("Test message")

                    assert result is None

    def test_print_debug_info_with_module_deployed(self) -> None:
        mock_module: MagicMock = MagicMock(spec=Path)
        with patch("acb.debug.get_calling_module", return_value=mock_module):
            with patch("acb.debug._deployed", True):
                with patch("acb.debug.patch_record") as mock_patch:
                    result: Any = print_debug_info("Test message")

                    mock_patch.assert_called_once_with(mock_module, "Test message")

                    assert result is None

    def test_print_debug_info_without_module(self) -> None:
        with patch("acb.debug.get_calling_module", return_value=None):
            result: Any = print_debug_info("Test message")

            assert result is None


class TestPprint:
    @pytest.mark.asyncio
    async def test_pprint(self) -> None:
        with patch("acb.debug.pformat") as mock_pformat:
            mock_pformat.return_value = "Formatted object"

            with patch("acb.debug.aprint") as mock_aprint:
                await pprint({"test": "object"})

                mock_pformat.assert_called_once_with({"test": "object"})

                mock_aprint.assert_called_once_with("Formatted object", use_stderr=True)

    @pytest.mark.asyncio
    async def test_pprint_with_custom_width(self) -> None:
        test_data = {"key": "value" * 20}
        with patch("acb.debug.pformat") as mock_pformat:
            with patch("acb.debug.aprint") as mock_aprint:
                await pprint(test_data)
                mock_pformat.assert_called_once_with(test_data)
                mock_aprint.assert_called_once()


# Note: TestInitDebug removed due to test isolation issues with parallel execution


class TestDebugGlobal:
    def test_debug_global(self) -> None:
        from icecream import ic

        assert debug is ic

    def test_debug_with_different_log_levels(self) -> None:
        test_messages = [
            "Test info message",
            "Test warning message",
            "Test error message",
        ]

        with patch.object(debug, "outputFunction") as mock_output_func:
            for message in test_messages:
                debug(message)
            assert mock_output_func.call_count == len(test_messages)

    def test_get_calling_module_from_debug(self) -> None:
        from acb.debug import get_calling_module

        mock_frame = MagicMock()
        mock_frame.f_back.f_back.f_back.f_code.co_filename = "/path/to/test_module.py"

        with patch("logging.currentframe", return_value=mock_frame):
            result = get_calling_module()
            assert result is None or isinstance(result, Path)


class TestInitDebugEnhancements:
    """Test enhanced error handling and warning suppression in init_debug."""

    def test_warning_suppression(self) -> None:
        """Test that icecream RuntimeWarnings are properly suppressed."""
        with patch("warnings.filterwarnings") as mock_filter_warnings:
            with patch("acb.debug.depends.get") as mock_depends_get:
                mock_config = MagicMock()
                mock_config.deployed = False
                mock_config.debug = MagicMock()
                mock_config.debug.production = False
                mock_depends_get.return_value = mock_config

                # Call init_debug to trigger warning suppression
                init_debug()

                # Verify that warnings.filterwarnings was called to suppress icecream warnings
                mock_filter_warnings.assert_any_call(
                    "ignore",
                    category=RuntimeWarning,
                    module="icecream",
                )

    def test_init_debug_with_config_available(self) -> None:
        """Test init_debug when config is available."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                mock_config = MagicMock()
                mock_config.deployed = False
                mock_config.debug = MagicMock()
                mock_config.debug.production = False
                mock_depends_get.return_value = mock_config

                init_debug()

                # Verify debug was configured
                mock_configure.assert_called()
                call_kwargs = mock_configure.call_args[1]
                assert "outputFunction" in call_kwargs
                assert "argToStringFunction" in call_kwargs
                assert call_kwargs["prefix"] == "    debug:  "
                assert call_kwargs["includeContext"]

    def test_init_debug_production_mode(self) -> None:
        """Test init_debug in production mode."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                mock_config = MagicMock()
                mock_config.deployed = True
                mock_config.debug = MagicMock()
                mock_config.debug.production = True
                mock_depends_get.return_value = mock_config

                init_debug()

                # Verify debug was configured for production
                mock_configure.assert_called()
                call_kwargs = mock_configure.call_args[1]
                assert call_kwargs["prefix"] == ""
                assert not call_kwargs["includeContext"]

    def test_init_debug_config_debug_production_flag(self) -> None:
        """Test init_debug respects config.debug.production flag."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                mock_config = MagicMock()
                mock_config.deployed = False  # Not deployed
                mock_config.debug = MagicMock()
                mock_config.debug.production = True  # But production flag is set
                mock_depends_get.return_value = mock_config

                init_debug()

                # Should use production settings due to debug.production flag
                mock_configure.assert_called()
                call_kwargs = mock_configure.call_args[1]
                assert call_kwargs["prefix"] == ""
                assert not call_kwargs["includeContext"]

    def test_init_debug_fallback_when_config_unavailable(self) -> None:
        """Test init_debug fallback configuration when config is not available."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                # Make depends.get raise an exception
                mock_depends_get.side_effect = Exception("Config not available")

                # Should not raise exception
                init_debug()

                # Should still configure debug with fallback settings
                mock_configure.assert_called()
                call_kwargs = mock_configure.call_args[1]
                assert "outputFunction" in call_kwargs
                assert "argToStringFunction" in call_kwargs
                assert call_kwargs["prefix"] == "    debug:  "
                assert call_kwargs["includeContext"]

    # Note: test_init_debug_exception_handling and test_debug_args_consistency
    # removed due to test isolation issues with parallel execution

    def test_get_calling_module_enhanced_error_handling(self) -> None:
        """Test enhanced error handling in get_calling_module."""
        from acb.debug import get_calling_module

        # Test with suppress context manager handling AttributeError
        with patch("logging.currentframe") as mock_frame:
            mock_frame.return_value = None
            result = get_calling_module()
            assert result is None

        # Test with suppress context manager handling TypeError
        with patch("logging.currentframe") as mock_frame:
            mock_frame.side_effect = TypeError("Frame error")
            result = get_calling_module()
            assert result is None

    def test_patch_record_enhanced_error_handling(self) -> None:
        """Test enhanced error handling in patch_record."""
        from acb.debug import patch_record

        mock_logger = MagicMock()
        mock_path = MagicMock(spec=Path)
        mock_path.name = "test_module"

        # Test normal operation
        patch_record(mock_path, "test message", logger=mock_logger)
        mock_logger.patch.assert_called_once()

        # Test with exception in logger operations
        mock_logger.reset_mock()
        mock_logger.patch.side_effect = Exception("Logger error")

        # Should not raise exception due to suppress context manager
        try:
            patch_record(mock_path, "test message", logger=mock_logger)
        except Exception as e:
            pytest.fail(f"patch_record should suppress exceptions, but raised: {e}")

    def test_colorized_stderr_print_enhanced_error_handling(self) -> None:
        """Test enhanced error handling in colorized_stderr_print."""
        # Test ImportError fallback
        with patch("acb.debug.colorize", side_effect=ImportError("No colorize")):
            with patch("builtins.print") as mock_print:
                with patch("sys.stderr") as mock_stderr:
                    colorized_stderr_print("test message")
                    mock_print.assert_called_once_with("test message", file=mock_stderr)

        # Test exception in asyncio.run fallback to print
        with patch("acb.debug.colorize", return_value="colored message"):
            with patch("acb.debug.supportTerminalColorsInWindows"):
                with patch(
                    "acb.debug.asyncio.run",
                    side_effect=Exception("Asyncio error"),
                ):
                    with patch("builtins.print") as mock_print:
                        with patch("sys.stderr") as mock_stderr:
                            colorized_stderr_print("test message")
                            mock_print.assert_called_once_with(
                                "colored message",
                                file=mock_stderr,
                            )

==> acb-0.31.1.tar.gz: acb-0.31.1/tests/test_debug.py <==
from pathlib import Path
from typing import Any, Final
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from acb.debug import (
    colorized_stderr_print,
    debug,
    init_debug,
    pprint,
    print_debug_info,
)
from acb.logger import Logger

TEST_MODULE_NAME: Final[str] = "test_module"
TEST_DEBUG_MSG: Final[str] = "test debug message"


@pytest.fixture
def mock_console() -> MagicMock:
    from rich.console import Console
    console = MagicMock(spec=Console)
    console.print = MagicMock()
    return console


@pytest.fixture
def mock_logger() -> MagicMock:
    logger = MagicMock(spec=Logger)
    logger.debug = MagicMock()
    return logger


class TestColorizedStderrPrint:
    def test_colorized_stderr_print_with_asyncio(self) -> None:
        with patch("acb.debug.colorize") as mock_colorize:
            mock_colorize.return_value = "Colored test message"

            with patch("acb.debug.supportTerminalColorsInWindows") as mock_support:
                mock_support.return_value.__enter__ = MagicMock()
                mock_support.return_value.__exit__ = MagicMock()

                mock_coro: AsyncMock = AsyncMock()
                with patch("acb.debug.aprint", return_value=mock_coro) as mock_aprint:

                    def mock_run_impl(coro: Any) -> None:
                        return None

                    with patch(
                        "acb.debug.asyncio.run",
                        side_effect=mock_run_impl,
                    ) as mock_run:
                        colorized_stderr_print("Test message")

                        mock_colorize.assert_called_once_with("Test message")

                        mock_support.assert_called_once()

                        mock_aprint.assert_called_once_with(
                            "Colored test message",
                            use_stderr=True,
                        )

                        mock_run.assert_called_once()

    def test_colorized_stderr_print_with_print(self) -> None:
        with patch("acb.debug.colorize") as mock_colorize:
            mock_colorize.return_value = "Colored test message"

            with patch("acb.debug.supportTerminalColorsInWindows") as mock_support:
                mock_support.return_value.__enter__ = MagicMock()
                mock_support.return_value.__exit__ = MagicMock()

                with patch(
                    "acb.debug.asyncio.run",
                    side_effect=Exception("Test exception"),
                ):
                    # Since the implementation doesn't fall back to print anymore,
                    # we just verify that no exception is raised
                    try:
                        colorized_stderr_print("Test message")
                    except Exception:
                        pytest.fail("colorized_stderr_print should not raise exceptions")

    def test_colorized_stderr_print_with_no_color_support(self) -> None:
        with patch("acb.debug.colorize", side_effect=ImportError):
            # Since the implementation suppresses ImportError,
            # we just verify that no exception is raised
            try:
                colorized_stderr_print("Test message")
            except Exception:
                pytest.fail("colorized_stderr_print should not raise exceptions")


class TestPrintDebugInfo:
    def test_print_debug_info_with_module_not_deployed(self) -> None:
        mock_module: MagicMock = MagicMock(spec=Path)
        with patch("acb.debug.get_calling_module", return_value=mock_module):
            with patch("acb.debug._deployed", False):
                with patch("acb.debug.colorized_stderr_print") as mock_print:
                    result: Any = print_debug_info("Test message")

                    mock_print.assert_called_once_with("Test message")

                    assert result is None

    def test_print_debug_info_with_module_deployed(self) -> None:
        mock_module: MagicMock = MagicMock(spec=Path)
        with patch("acb.debug.get_calling_module", return_value=mock_module):
            with patch("acb.debug._deployed", True):
                with patch("acb.debug.patch_record") as mock_patch:
                    result: Any = print_debug_info("Test message")

                    mock_patch.assert_called_once_with(mock_module, "Test message")

                    assert result is None

    def test_print_debug_info_without_module(self) -> None:
        with patch("acb.debug.get_calling_module", return_value=None):
            result: Any = print_debug_info("Test message")

            assert result is None


class TestPprint:
    @pytest.mark.asyncio
    async def test_pprint(self) -> None:
        with patch("acb.debug._pformat") as mock_pformat:
            mock_pformat.return_value = "Formatted object"

            with patch("acb.debug.aprint") as mock_aprint:
                await pprint({"test": "object"})

                mock_pformat.assert_called_once_with({"test": "object"})

                mock_aprint.assert_called_once_with("Formatted object", use_stderr=True)

    @pytest.mark.asyncio
    async def test_pprint_with_custom_width(self) -> None:
        test_data = {"key": "value" * 20}
        with patch("acb.debug._pformat") as mock_pformat:
            with patch("acb.debug.aprint") as mock_aprint:
                await pprint(test_data)
                mock_pformat.assert_called_once_with(test_data)
                mock_aprint.assert_called_once()


# Note: TestInitDebug removed due to test isolation issues with parallel execution


class TestDebugGlobal:
    def test_debug_global(self) -> None:
        from icecream import ic

        assert debug is ic

    def test_debug_with_different_log_levels(self) -> None:
        test_messages = [
            "Test info message",
            "Test warning message",
            "Test error message",
        ]

        with patch.object(debug, "outputFunction") as mock_output_func:
            for message in test_messages:
                debug(message)
            assert mock_output_func.call_count == len(test_messages)

    def test_get_calling_module_from_debug(self) -> None:
        from acb.debug import get_calling_module

        mock_frame = MagicMock()
        mock_frame.f_back.f_back.f_back.f_code.co_filename = "/path/to/test_module.py"

        with patch("logging.currentframe", return_value=mock_frame):
            result = get_calling_module()
            assert result is None or isinstance(result, Path)


class TestInitDebugEnhancements:
    """Test enhanced error handling and warning suppression in init_debug."""

    def test_warning_suppression(self) -> None:
        """Test that icecream RuntimeWarnings are properly suppressed."""
        with patch("warnings.filterwarnings") as mock_filter_warnings:
            with patch("acb.debug.depends.get") as mock_depends_get:
                mock_config = MagicMock()
                mock_config.deployed = False
                mock_config.debug = MagicMock()
                mock_config.debug.production = False
                mock_depends_get.return_value = mock_config

                # Call init_debug to trigger warning suppression
                init_debug()

                # Verify that warnings.filterwarnings was called to suppress icecream warnings
                mock_filter_warnings.assert_any_call(
                    "ignore",
                    category=RuntimeWarning,
                    module="icecream",
                )

    def test_init_debug_with_config_available(self) -> None:
        """Test init_debug when config is available."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                # Create a proper mock config structure
                mock_config = MagicMock()
                mock_config.deployed = False
                # Create a mock debug attribute
                mock_debug = MagicMock()
                mock_debug.production = False
                mock_config.debug = mock_debug
                mock_depends_get.return_value = mock_config

                init_debug()

                # Verify debug was configured
                assert mock_configure.called
                # Check the LAST call's arguments (final configuration)
                last_call_kwargs = mock_configure.call_args_list[-1][1]
                assert "outputFunction" in last_call_kwargs
                assert "argToStringFunction" in last_call_kwargs
                assert last_call_kwargs["prefix"] == "    debug:  "
                assert last_call_kwargs["includeContext"] is True

    def test_init_debug_production_mode(self) -> None:
        """Test init_debug in production mode."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                # Create a proper mock config structure
                mock_config = MagicMock()
                mock_config.deployed = True
                # Create a mock debug attribute with production property
                mock_debug = MagicMock()
                mock_debug.production = True
                mock_config.debug = mock_debug
                mock_depends_get.return_value = mock_config

                init_debug()

                # Verify debug was configured (might be called multiple times)
                assert mock_configure.called
                # Check the LAST call's arguments (production settings)
                last_call_kwargs = mock_configure.call_args_list[-1][1]
                assert last_call_kwargs["prefix"] == ""
                assert last_call_kwargs["includeContext"] is False

    def test_init_debug_config_debug_production_flag(self) -> None:
        """Test init_debug respects config.debug.production flag."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                # Create a proper mock config structure
                mock_config = MagicMock()
                mock_config.deployed = False  # Not deployed
                # Create a mock debug attribute with production property
                mock_debug = MagicMock()
                mock_debug.production = True  # But production flag is set
                mock_config.debug = mock_debug
                mock_depends_get.return_value = mock_config

                init_debug()

                # Should use production settings due to debug.production flag
                # Verify debug was configured (might be called multiple times)
                assert mock_configure.called
                # Check the LAST call's arguments (production settings)
                last_call_kwargs = mock_configure.call_args_list[-1][1]
                assert last_call_kwargs["prefix"] == ""
                assert last_call_kwargs["includeContext"] is False

    def test_init_debug_fallback_when_config_unavailable(self) -> None:
        """Test init_debug fallback configuration when config is not available."""
        with patch("acb.debug.depends.get") as mock_depends_get:
            with patch.object(debug, "configureOutput") as mock_configure:
                # Make depends.get raise an exception
                mock_depends_get.side_effect = Exception("Config not available")

                # Should not raise exception
                init_debug()

                # Should still configure debug with fallback settings
                assert mock_configure.called
                # Check the LAST call's arguments (final configuration)
                last_call_kwargs = mock_configure.call_args_list[-1][1]
                assert "outputFunction" in last_call_kwargs
                assert "argToStringFunction" in last_call_kwargs
                assert last_call_kwargs["prefix"] == "    debug:  "
                assert last_call_kwargs["includeContext"] is True

    # Note: test_init_debug_exception_handling and test_debug_args_consistency
    # removed due to test isolation issues with parallel execution

    def test_get_calling_module_enhanced_error_handling(self) -> None:
        """Test enhanced error handling in get_calling_module."""
        from acb.debug import get_calling_module

        # Test with suppress context manager handling AttributeError
        with patch("logging.currentframe") as mock_frame:
            mock_frame.return_value = None
            result = get_calling_module()
            assert result is None

        # Test with suppress context manager handling TypeError
        with patch("logging.currentframe") as mock_frame:
            mock_frame.side_effect = TypeError("Frame error")
            result = get_calling_module()
            assert result is None

    def test_patch_record_enhanced_error_handling(self) -> None:
        """Test enhanced error handling in patch_record."""
        from acb.debug import patch_record

        mock_logger = MagicMock()
        mock_path = MagicMock(spec=Path)
        mock_path.name = "test_module"

        # Test normal operation
        patch_record(mock_path, "test message", logger=mock_logger)
        mock_logger.patch.assert_called_once()

        # Test with exception in logger operations
        mock_logger.reset_mock()
        mock_logger.patch.side_effect = Exception("Logger error")

        # Should not raise exception due to suppress context manager
        try:
            patch_record(mock_path, "test message", logger=mock_logger)
        except Exception as e:
            pytest.fail(f"patch_record should suppress exceptions, but raised: {e}")

    def test_colorized_stderr_print_enhanced_error_handling(self) -> None:
        """Test enhanced error handling in colorized_stderr_print."""
        # Test ImportError fallback - function should suppress the error
        with patch("acb.debug.colorize", side_effect=ImportError("No colorize")):
            # Should not raise exception due to suppress context manager
            try:
                colorized_stderr_print("test message")
            except Exception as e:
                pytest.fail(f"colorized_stderr_print should suppress ImportError, but raised: {e}")

        # Test exception in asyncio.run - function should suppress the error
        with patch("acb.debug.colorize", return_value="colored message"):
            with patch("acb.debug.supportTerminalColorsInWindows"):
                with patch(
                    "acb.debug.asyncio.run",
                    side_effect=Exception("Asyncio error"),
                ):
                    # Should not raise exception due to suppress context manager
                    try:
                        colorized_stderr_print("test message")
                    except Exception as e:
                        pytest.fail(f"colorized_stderr_print should suppress asyncio errors, but raised: {e}")
